# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: proto/command.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'proto/command.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import response_pb2 as proto_dot_response__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13proto/command.proto\x12\x15\x66\x61teark.proto.command\x1a\x14proto/response.proto\"#\n\x14SendWOCommandRequest\x12\x0b\n\x03\x63md\x18\x01 \x01(\t\"#\n\x14SendWSCommandRequest\x12\x0b\n\x03\x63md\x18\x01 \x01(\t\"\'\n\x18SendPlayerCommandRequest\x12\x0b\n\x03\x63md\x18\x01 \x01(\t\"7\n\x14SendAICommandRequest\x12\x12\n\nruntime_id\x18\x01 \x01(\t\x12\x0b\n\x03\x63md\x18\x02 \x01(\t\"/\n SendWSCommandWithResponseRequest\x12\x0b\n\x03\x63md\x18\x01 \x01(\t\"3\n$SendPlayerCommandWithResponseRequest\x12\x0b\n\x03\x63md\x18\x01 \x01(\t\"C\n SendAICommandWithResponseRequest\x12\x12\n\nruntime_id\x18\x01 \x01(\t\x12\x0b\n\x03\x63md\x18\x02 \x01(\t2\xba\x06\n\x0e\x43ommandService\x12\x65\n\rSendWOCommand\x12+.fateark.proto.command.SendWOCommandRequest\x1a\'.fateark.proto.response.GeneralResponse\x12\x65\n\rSendWSCommand\x12+.fateark.proto.command.SendWSCommandRequest\x1a\'.fateark.proto.response.GeneralResponse\x12m\n\x11SendPlayerCommand\x12/.fateark.proto.command.SendPlayerCommandRequest\x1a\'.fateark.proto.response.GeneralResponse\x12\x65\n\rSendAICommand\x12+.fateark.proto.command.SendAICommandRequest\x1a\'.fateark.proto.response.GeneralResponse\x12}\n\x19SendWSCommandWithResponse\x12\x37.fateark.proto.command.SendWSCommandWithResponseRequest\x1a\'.fateark.proto.response.GeneralResponse\x12\x85\x01\n\x1dSendPlayerCommandWithResponse\x12;.fateark.proto.command.SendPlayerCommandWithResponseRequest\x1a\'.fateark.proto.response.GeneralResponse\x12}\n\x19SendAICommandWithResponse\x12\x37.fateark.proto.command.SendAICommandWithResponseRequest\x1a\'.fateark.proto.response.GeneralResponseB\x1fZ\x1d./proto/gen/command;commandpbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'proto.command_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z\035./proto/gen/command;commandpb'
  _globals['_SENDWOCOMMANDREQUEST']._serialized_start=68
  _globals['_SENDWOCOMMANDREQUEST']._serialized_end=103
  _globals['_SENDWSCOMMANDREQUEST']._serialized_start=105
  _globals['_SENDWSCOMMANDREQUEST']._serialized_end=140
  _globals['_SENDPLAYERCOMMANDREQUEST']._serialized_start=142
  _globals['_SENDPLAYERCOMMANDREQUEST']._serialized_end=181
  _globals['_SENDAICOMMANDREQUEST']._serialized_start=183
  _globals['_SENDAICOMMANDREQUEST']._serialized_end=238
  _globals['_SENDWSCOMMANDWITHRESPONSEREQUEST']._serialized_start=240
  _globals['_SENDWSCOMMANDWITHRESPONSEREQUEST']._serialized_end=287
  _globals['_SENDPLAYERCOMMANDWITHRESPONSEREQUEST']._serialized_start=289
  _globals['_SENDPLAYERCOMMANDWITHRESPONSEREQUEST']._serialized_end=340
  _globals['_SENDAICOMMANDWITHRESPONSEREQUEST']._serialized_start=342
  _globals['_SENDAICOMMANDWITHRESPONSEREQUEST']._serialized_end=409
  _globals['_COMMANDSERVICE']._serialized_start=412
  _globals['_COMMANDSERVICE']._serialized_end=1238
# @@protoc_insertion_point(module_scope)
