# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from . import playerkit_pb2 as proto_dot_playerkit__pb2
from . import response_pb2 as proto_dot_response__pb2

GRPC_GENERATED_VERSION = '1.71.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in proto/playerkit_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class PlayerKitServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetAllOnlinePlayers = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetAllOnlinePlayers',
                request_serializer=proto_dot_playerkit__pb2.GetAllOnlinePlayersRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerByName = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerByName',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerByNameRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerByUUID = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerByUUID',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerByUUIDRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.ReleaseBindPlayer = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/ReleaseBindPlayer',
                request_serializer=proto_dot_playerkit__pb2.ReleaseBindPlayerRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerName = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerName',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerNameRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerEntityUniqueID = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerEntityUniqueID',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerEntityUniqueIDRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralInt64Response.FromString,
                _registered_method=True)
        self.GetPlayerLoginTime = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerLoginTime',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerLoginTimeRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralInt64Response.FromString,
                _registered_method=True)
        self.GetPlayerPlatformChatID = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerPlatformChatID',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerPlatformChatIDRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerBuildPlatform = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerBuildPlatform',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerBuildPlatformRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralInt32Response.FromString,
                _registered_method=True)
        self.GetPlayerSkinID = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerSkinID',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerSkinIDRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerCanBuild = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanBuild',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerCanBuildRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.SetPlayerCanBuild = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanBuild',
                request_serializer=proto_dot_playerkit__pb2.SetPlayerCanBuildRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerCanDig = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanDig',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerCanDigRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.SetPlayerCanDig = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanDig',
                request_serializer=proto_dot_playerkit__pb2.SetPlayerCanDigRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerCanDoorsAndSwitches = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanDoorsAndSwitches',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerCanDoorsAndSwitchesRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.SetPlayerCanDoorsAndSwitches = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanDoorsAndSwitches',
                request_serializer=proto_dot_playerkit__pb2.SetPlayerCanDoorsAndSwitchesRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerCanOpenContainers = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanOpenContainers',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerCanOpenContainersRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.SetPlayerCanOpenContainers = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanOpenContainers',
                request_serializer=proto_dot_playerkit__pb2.SetPlayerCanOpenContainersRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerCanAttackPlayers = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanAttackPlayers',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerCanAttackPlayersRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.SetPlayerCanAttackPlayers = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanAttackPlayers',
                request_serializer=proto_dot_playerkit__pb2.SetPlayerCanAttackPlayersRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerCanAttackMobs = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanAttackMobs',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerCanAttackMobsRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.SetPlayerCanAttackMobs = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanAttackMobs',
                request_serializer=proto_dot_playerkit__pb2.SetPlayerCanAttackMobsRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerCanOperatorCommands = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanOperatorCommands',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerCanOperatorCommandsRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.SetPlayerCanOperatorCommands = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanOperatorCommands',
                request_serializer=proto_dot_playerkit__pb2.SetPlayerCanOperatorCommandsRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerCanTeleport = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanTeleport',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerCanTeleportRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.SetPlayerCanTeleport = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanTeleport',
                request_serializer=proto_dot_playerkit__pb2.SetPlayerCanTeleportRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.GetPlayerStatusInvulnerable = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerStatusInvulnerable',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerStatusInvulnerableRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.GetPlayerStatusFlying = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerStatusFlying',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerStatusFlyingRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.GetPlayerStatusMayFly = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerStatusMayFly',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerStatusMayFlyRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.GetPlayerDeviceID = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerDeviceID',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerDeviceIDRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerEntityRuntimeID = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerEntityRuntimeID',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerEntityRuntimeIDRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralUint64Response.FromString,
                _registered_method=True)
        self.GetPlayerEntityMetadata = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerEntityMetadata',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerEntityMetadataRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.GetPlayerIsOP = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerIsOP',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerIsOPRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.GetPlayerOnline = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/GetPlayerOnline',
                request_serializer=proto_dot_playerkit__pb2.GetPlayerOnlineRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralBoolResponse.FromString,
                _registered_method=True)
        self.SendPlayerChat = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SendPlayerChat',
                request_serializer=proto_dot_playerkit__pb2.SendPlayerChatRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.SendPlayerRawChat = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SendPlayerRawChat',
                request_serializer=proto_dot_playerkit__pb2.SendPlayerRawChatRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.SendPlayerTitle = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SendPlayerTitle',
                request_serializer=proto_dot_playerkit__pb2.SendPlayerTitleRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.SendPlayerActionBar = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/SendPlayerActionBar',
                request_serializer=proto_dot_playerkit__pb2.SendPlayerActionBarRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)
        self.InterceptPlayerJustNextInput = channel.unary_unary(
                '/fateark.proto.playerkit.PlayerKitService/InterceptPlayerJustNextInput',
                request_serializer=proto_dot_playerkit__pb2.InterceptPlayerJustNextInputRequest.SerializeToString,
                response_deserializer=proto_dot_response__pb2.GeneralResponse.FromString,
                _registered_method=True)


class PlayerKitServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetAllOnlinePlayers(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerByName(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerByUUID(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReleaseBindPlayer(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerName(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerEntityUniqueID(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerLoginTime(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerPlatformChatID(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerBuildPlatform(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerSkinID(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerCanBuild(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetPlayerCanBuild(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerCanDig(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetPlayerCanDig(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerCanDoorsAndSwitches(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetPlayerCanDoorsAndSwitches(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerCanOpenContainers(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetPlayerCanOpenContainers(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerCanAttackPlayers(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetPlayerCanAttackPlayers(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerCanAttackMobs(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetPlayerCanAttackMobs(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerCanOperatorCommands(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetPlayerCanOperatorCommands(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerCanTeleport(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetPlayerCanTeleport(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerStatusInvulnerable(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerStatusFlying(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerStatusMayFly(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerDeviceID(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerEntityRuntimeID(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerEntityMetadata(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerIsOP(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPlayerOnline(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SendPlayerChat(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SendPlayerRawChat(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SendPlayerTitle(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SendPlayerActionBar(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def InterceptPlayerJustNextInput(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_PlayerKitServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetAllOnlinePlayers': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAllOnlinePlayers,
                    request_deserializer=proto_dot_playerkit__pb2.GetAllOnlinePlayersRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerByName': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerByName,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerByNameRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerByUUID': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerByUUID,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerByUUIDRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'ReleaseBindPlayer': grpc.unary_unary_rpc_method_handler(
                    servicer.ReleaseBindPlayer,
                    request_deserializer=proto_dot_playerkit__pb2.ReleaseBindPlayerRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerName': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerName,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerNameRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerEntityUniqueID': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerEntityUniqueID,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerEntityUniqueIDRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralInt64Response.SerializeToString,
            ),
            'GetPlayerLoginTime': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerLoginTime,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerLoginTimeRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralInt64Response.SerializeToString,
            ),
            'GetPlayerPlatformChatID': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerPlatformChatID,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerPlatformChatIDRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerBuildPlatform': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerBuildPlatform,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerBuildPlatformRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralInt32Response.SerializeToString,
            ),
            'GetPlayerSkinID': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerSkinID,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerSkinIDRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerCanBuild': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerCanBuild,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerCanBuildRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'SetPlayerCanBuild': grpc.unary_unary_rpc_method_handler(
                    servicer.SetPlayerCanBuild,
                    request_deserializer=proto_dot_playerkit__pb2.SetPlayerCanBuildRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerCanDig': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerCanDig,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerCanDigRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'SetPlayerCanDig': grpc.unary_unary_rpc_method_handler(
                    servicer.SetPlayerCanDig,
                    request_deserializer=proto_dot_playerkit__pb2.SetPlayerCanDigRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerCanDoorsAndSwitches': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerCanDoorsAndSwitches,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerCanDoorsAndSwitchesRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'SetPlayerCanDoorsAndSwitches': grpc.unary_unary_rpc_method_handler(
                    servicer.SetPlayerCanDoorsAndSwitches,
                    request_deserializer=proto_dot_playerkit__pb2.SetPlayerCanDoorsAndSwitchesRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerCanOpenContainers': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerCanOpenContainers,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerCanOpenContainersRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'SetPlayerCanOpenContainers': grpc.unary_unary_rpc_method_handler(
                    servicer.SetPlayerCanOpenContainers,
                    request_deserializer=proto_dot_playerkit__pb2.SetPlayerCanOpenContainersRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerCanAttackPlayers': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerCanAttackPlayers,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerCanAttackPlayersRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'SetPlayerCanAttackPlayers': grpc.unary_unary_rpc_method_handler(
                    servicer.SetPlayerCanAttackPlayers,
                    request_deserializer=proto_dot_playerkit__pb2.SetPlayerCanAttackPlayersRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerCanAttackMobs': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerCanAttackMobs,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerCanAttackMobsRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'SetPlayerCanAttackMobs': grpc.unary_unary_rpc_method_handler(
                    servicer.SetPlayerCanAttackMobs,
                    request_deserializer=proto_dot_playerkit__pb2.SetPlayerCanAttackMobsRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerCanOperatorCommands': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerCanOperatorCommands,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerCanOperatorCommandsRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'SetPlayerCanOperatorCommands': grpc.unary_unary_rpc_method_handler(
                    servicer.SetPlayerCanOperatorCommands,
                    request_deserializer=proto_dot_playerkit__pb2.SetPlayerCanOperatorCommandsRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerCanTeleport': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerCanTeleport,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerCanTeleportRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'SetPlayerCanTeleport': grpc.unary_unary_rpc_method_handler(
                    servicer.SetPlayerCanTeleport,
                    request_deserializer=proto_dot_playerkit__pb2.SetPlayerCanTeleportRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'GetPlayerStatusInvulnerable': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerStatusInvulnerable,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerStatusInvulnerableRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'GetPlayerStatusFlying': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerStatusFlying,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerStatusFlyingRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'GetPlayerStatusMayFly': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerStatusMayFly,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerStatusMayFlyRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'GetPlayerDeviceID': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerDeviceID,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerDeviceIDRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerEntityRuntimeID': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerEntityRuntimeID,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerEntityRuntimeIDRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralUint64Response.SerializeToString,
            ),
            'GetPlayerEntityMetadata': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerEntityMetadata,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerEntityMetadataRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'GetPlayerIsOP': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerIsOP,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerIsOPRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'GetPlayerOnline': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPlayerOnline,
                    request_deserializer=proto_dot_playerkit__pb2.GetPlayerOnlineRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralBoolResponse.SerializeToString,
            ),
            'SendPlayerChat': grpc.unary_unary_rpc_method_handler(
                    servicer.SendPlayerChat,
                    request_deserializer=proto_dot_playerkit__pb2.SendPlayerChatRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'SendPlayerRawChat': grpc.unary_unary_rpc_method_handler(
                    servicer.SendPlayerRawChat,
                    request_deserializer=proto_dot_playerkit__pb2.SendPlayerRawChatRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'SendPlayerTitle': grpc.unary_unary_rpc_method_handler(
                    servicer.SendPlayerTitle,
                    request_deserializer=proto_dot_playerkit__pb2.SendPlayerTitleRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'SendPlayerActionBar': grpc.unary_unary_rpc_method_handler(
                    servicer.SendPlayerActionBar,
                    request_deserializer=proto_dot_playerkit__pb2.SendPlayerActionBarRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
            'InterceptPlayerJustNextInput': grpc.unary_unary_rpc_method_handler(
                    servicer.InterceptPlayerJustNextInput,
                    request_deserializer=proto_dot_playerkit__pb2.InterceptPlayerJustNextInputRequest.FromString,
                    response_serializer=proto_dot_response__pb2.GeneralResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'fateark.proto.playerkit.PlayerKitService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('fateark.proto.playerkit.PlayerKitService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class PlayerKitService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetAllOnlinePlayers(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetAllOnlinePlayers',
            proto_dot_playerkit__pb2.GetAllOnlinePlayersRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerByName(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerByName',
            proto_dot_playerkit__pb2.GetPlayerByNameRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerByUUID(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerByUUID',
            proto_dot_playerkit__pb2.GetPlayerByUUIDRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ReleaseBindPlayer(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/ReleaseBindPlayer',
            proto_dot_playerkit__pb2.ReleaseBindPlayerRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerName(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerName',
            proto_dot_playerkit__pb2.GetPlayerNameRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerEntityUniqueID(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerEntityUniqueID',
            proto_dot_playerkit__pb2.GetPlayerEntityUniqueIDRequest.SerializeToString,
            proto_dot_response__pb2.GeneralInt64Response.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerLoginTime(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerLoginTime',
            proto_dot_playerkit__pb2.GetPlayerLoginTimeRequest.SerializeToString,
            proto_dot_response__pb2.GeneralInt64Response.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerPlatformChatID(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerPlatformChatID',
            proto_dot_playerkit__pb2.GetPlayerPlatformChatIDRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerBuildPlatform(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerBuildPlatform',
            proto_dot_playerkit__pb2.GetPlayerBuildPlatformRequest.SerializeToString,
            proto_dot_response__pb2.GeneralInt32Response.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerSkinID(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerSkinID',
            proto_dot_playerkit__pb2.GetPlayerSkinIDRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerCanBuild(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanBuild',
            proto_dot_playerkit__pb2.GetPlayerCanBuildRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SetPlayerCanBuild(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanBuild',
            proto_dot_playerkit__pb2.SetPlayerCanBuildRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerCanDig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanDig',
            proto_dot_playerkit__pb2.GetPlayerCanDigRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SetPlayerCanDig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanDig',
            proto_dot_playerkit__pb2.SetPlayerCanDigRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerCanDoorsAndSwitches(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanDoorsAndSwitches',
            proto_dot_playerkit__pb2.GetPlayerCanDoorsAndSwitchesRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SetPlayerCanDoorsAndSwitches(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanDoorsAndSwitches',
            proto_dot_playerkit__pb2.SetPlayerCanDoorsAndSwitchesRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerCanOpenContainers(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanOpenContainers',
            proto_dot_playerkit__pb2.GetPlayerCanOpenContainersRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SetPlayerCanOpenContainers(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanOpenContainers',
            proto_dot_playerkit__pb2.SetPlayerCanOpenContainersRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerCanAttackPlayers(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanAttackPlayers',
            proto_dot_playerkit__pb2.GetPlayerCanAttackPlayersRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SetPlayerCanAttackPlayers(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanAttackPlayers',
            proto_dot_playerkit__pb2.SetPlayerCanAttackPlayersRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerCanAttackMobs(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanAttackMobs',
            proto_dot_playerkit__pb2.GetPlayerCanAttackMobsRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SetPlayerCanAttackMobs(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanAttackMobs',
            proto_dot_playerkit__pb2.SetPlayerCanAttackMobsRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerCanOperatorCommands(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanOperatorCommands',
            proto_dot_playerkit__pb2.GetPlayerCanOperatorCommandsRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SetPlayerCanOperatorCommands(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanOperatorCommands',
            proto_dot_playerkit__pb2.SetPlayerCanOperatorCommandsRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerCanTeleport(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerCanTeleport',
            proto_dot_playerkit__pb2.GetPlayerCanTeleportRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SetPlayerCanTeleport(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SetPlayerCanTeleport',
            proto_dot_playerkit__pb2.SetPlayerCanTeleportRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerStatusInvulnerable(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerStatusInvulnerable',
            proto_dot_playerkit__pb2.GetPlayerStatusInvulnerableRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerStatusFlying(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerStatusFlying',
            proto_dot_playerkit__pb2.GetPlayerStatusFlyingRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerStatusMayFly(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerStatusMayFly',
            proto_dot_playerkit__pb2.GetPlayerStatusMayFlyRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerDeviceID(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerDeviceID',
            proto_dot_playerkit__pb2.GetPlayerDeviceIDRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerEntityRuntimeID(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerEntityRuntimeID',
            proto_dot_playerkit__pb2.GetPlayerEntityRuntimeIDRequest.SerializeToString,
            proto_dot_response__pb2.GeneralUint64Response.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerEntityMetadata(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerEntityMetadata',
            proto_dot_playerkit__pb2.GetPlayerEntityMetadataRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerIsOP(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerIsOP',
            proto_dot_playerkit__pb2.GetPlayerIsOPRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPlayerOnline(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/GetPlayerOnline',
            proto_dot_playerkit__pb2.GetPlayerOnlineRequest.SerializeToString,
            proto_dot_response__pb2.GeneralBoolResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SendPlayerChat(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SendPlayerChat',
            proto_dot_playerkit__pb2.SendPlayerChatRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SendPlayerRawChat(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SendPlayerRawChat',
            proto_dot_playerkit__pb2.SendPlayerRawChatRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SendPlayerTitle(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SendPlayerTitle',
            proto_dot_playerkit__pb2.SendPlayerTitleRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SendPlayerActionBar(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/SendPlayerActionBar',
            proto_dot_playerkit__pb2.SendPlayerActionBarRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def InterceptPlayerJustNextInput(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fateark.proto.playerkit.PlayerKitService/InterceptPlayerJustNextInput',
            proto_dot_playerkit__pb2.InterceptPlayerJustNextInputRequest.SerializeToString,
            proto_dot_response__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)
