# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: proto/listener.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'proto/listener.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import response_pb2 as proto_dot_response__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14proto/listener.proto\x12\x16\x66\x61teark.proto.listener\x1a\x14proto/response.proto\"\x16\n\x14ListenFateArkRequest\"\x16\n\x14ListenPacketsRequest\"-\n\x18ListenTypedPacketRequest\x12\x11\n\tpacket_id\x18\x01 \x01(\r\"\x1b\n\x19ListenPlayerChangeRequest\"\x13\n\x11ListenChatRequest\")\n\x19ListenCommandBlockRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"8\n\x06Output\x12\x10\n\x08msg_type\x18\x01 \x01(\t\x12\x0b\n\x03msg\x18\x02 \x01(\t\x12\x0f\n\x07\x65rr_msg\x18\x03 \x01(\t\"%\n\x06Packet\x12\n\n\x02id\x18\x01 \x01(\r\x12\x0f\n\x07payload\x18\x02 \x01(\t\"\x1e\n\x0cPlayerAction\x12\x0e\n\x06\x61\x63tion\x18\x01 \x01(\t\"\x17\n\x04\x43hat\x12\x0f\n\x07payload\x18\x01 \x01(\t2\xf6\x04\n\x0fListenerService\x12_\n\rListenFateArk\x12,.fateark.proto.listener.ListenFateArkRequest\x1a\x1e.fateark.proto.listener.Output0\x01\x12_\n\rListenPackets\x12,.fateark.proto.listener.ListenPacketsRequest\x1a\x1e.fateark.proto.listener.Packet0\x01\x12n\n\x11ListenTypedPacket\x12\x30.fateark.proto.listener.ListenTypedPacketRequest\x1a\'.fateark.proto.response.GeneralResponse\x12o\n\x12ListenPlayerChange\x12\x31.fateark.proto.listener.ListenPlayerChangeRequest\x1a$.fateark.proto.listener.PlayerAction0\x01\x12W\n\nListenChat\x12).fateark.proto.listener.ListenChatRequest\x1a\x1c.fateark.proto.listener.Chat0\x01\x12g\n\x12ListenCommandBlock\x12\x31.fateark.proto.listener.ListenCommandBlockRequest\x1a\x1c.fateark.proto.listener.Chat0\x01\x42!Z\x1f./proto/gen/listener;listenerpbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'proto.listener_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z\037./proto/gen/listener;listenerpb'
  _globals['_LISTENFATEARKREQUEST']._serialized_start=70
  _globals['_LISTENFATEARKREQUEST']._serialized_end=92
  _globals['_LISTENPACKETSREQUEST']._serialized_start=94
  _globals['_LISTENPACKETSREQUEST']._serialized_end=116
  _globals['_LISTENTYPEDPACKETREQUEST']._serialized_start=118
  _globals['_LISTENTYPEDPACKETREQUEST']._serialized_end=163
  _globals['_LISTENPLAYERCHANGEREQUEST']._serialized_start=165
  _globals['_LISTENPLAYERCHANGEREQUEST']._serialized_end=192
  _globals['_LISTENCHATREQUEST']._serialized_start=194
  _globals['_LISTENCHATREQUEST']._serialized_end=213
  _globals['_LISTENCOMMANDBLOCKREQUEST']._serialized_start=215
  _globals['_LISTENCOMMANDBLOCKREQUEST']._serialized_end=256
  _globals['_OUTPUT']._serialized_start=258
  _globals['_OUTPUT']._serialized_end=314
  _globals['_PACKET']._serialized_start=316
  _globals['_PACKET']._serialized_end=353
  _globals['_PLAYERACTION']._serialized_start=355
  _globals['_PLAYERACTION']._serialized_end=385
  _globals['_CHAT']._serialized_start=387
  _globals['_CHAT']._serialized_end=410
  _globals['_LISTENERSERVICE']._serialized_start=413
  _globals['_LISTENERSERVICE']._serialized_end=1043
# @@protoc_insertion_point(module_scope)
