# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: proto/utils.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'proto/utils.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import response_pb2 as proto_dot_response__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11proto/utils.proto\x12\x13\x66\x61teark.proto.utils\x1a\x14proto/response.proto\"8\n\x11SendPacketRequest\x12\x11\n\tpacket_id\x18\x01 \x01(\x05\x12\x10\n\x08json_str\x18\x02 \x01(\t\"\x1f\n\x1dGetPacketNameIDMappingRequest\"(\n&GetClientMaintainedBotBasicInfoRequest\"&\n$GetClientMaintainedExtendInfoRequest2\xf4\x03\n\x0cUtilsService\x12]\n\nSendPacket\x12&.fateark.proto.utils.SendPacketRequest\x1a\'.fateark.proto.response.GeneralResponse\x12u\n\x16GetPacketNameIDMapping\x12\x32.fateark.proto.utils.GetPacketNameIDMappingRequest\x1a\'.fateark.proto.response.GeneralResponse\x12\x87\x01\n\x1fGetClientMaintainedBotBasicInfo\x12;.fateark.proto.utils.GetClientMaintainedBotBasicInfoRequest\x1a\'.fateark.proto.response.GeneralResponse\x12\x83\x01\n\x1dGetClientMaintainedExtendInfo\x12\x39.fateark.proto.utils.GetClientMaintainedExtendInfoRequest\x1a\'.fateark.proto.response.GeneralResponseB\x1bZ\x19./proto/gen/utils;utilspbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'proto.utils_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z\031./proto/gen/utils;utilspb'
  _globals['_SENDPACKETREQUEST']._serialized_start=64
  _globals['_SENDPACKETREQUEST']._serialized_end=120
  _globals['_GETPACKETNAMEIDMAPPINGREQUEST']._serialized_start=122
  _globals['_GETPACKETNAMEIDMAPPINGREQUEST']._serialized_end=153
  _globals['_GETCLIENTMAINTAINEDBOTBASICINFOREQUEST']._serialized_start=155
  _globals['_GETCLIENTMAINTAINEDBOTBASICINFOREQUEST']._serialized_end=195
  _globals['_GETCLIENTMAINTAINEDEXTENDINFOREQUEST']._serialized_start=197
  _globals['_GETCLIENTMAINTAINEDEXTENDINFOREQUEST']._serialized_end=235
  _globals['_UTILSSERVICE']._serialized_start=238
  _globals['_UTILSSERVICE']._serialized_end=738
# @@protoc_insertion_point(module_scope)
