# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: proto/response.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'proto/response.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14proto/response.proto\x12\x16\x66\x61teark.proto.response\"\x98\x01\n\x0fGeneralResponse\x12>\n\x06status\x18\x01 \x01(\x0e\x32..fateark.proto.response.GeneralResponse.Status\x12\x0f\n\x07payload\x18\x02 \x01(\t\x12\x11\n\terror_msg\x18\x03 \x01(\t\"!\n\x06Status\x12\x0b\n\x07SUCCESS\x10\x00\x12\n\n\x06\x46\x41ILED\x10\x01\"\xa2\x01\n\x14GeneralInt32Response\x12\x43\n\x06status\x18\x01 \x01(\x0e\x32\x33.fateark.proto.response.GeneralInt32Response.Status\x12\x0f\n\x07payload\x18\x02 \x01(\x05\x12\x11\n\terror_msg\x18\x03 \x01(\t\"!\n\x06Status\x12\x0b\n\x07SUCCESS\x10\x00\x12\n\n\x06\x46\x41ILED\x10\x01\"\xa2\x01\n\x14GeneralInt64Response\x12\x43\n\x06status\x18\x01 \x01(\x0e\x32\x33.fateark.proto.response.GeneralInt64Response.Status\x12\x0f\n\x07payload\x18\x02 \x01(\x03\x12\x11\n\terror_msg\x18\x03 \x01(\t\"!\n\x06Status\x12\x0b\n\x07SUCCESS\x10\x00\x12\n\n\x06\x46\x41ILED\x10\x01\"\xa4\x01\n\x15GeneralUint64Response\x12\x44\n\x06status\x18\x01 \x01(\x0e\x32\x34.fateark.proto.response.GeneralUint64Response.Status\x12\x0f\n\x07payload\x18\x02 \x01(\x04\x12\x11\n\terror_msg\x18\x03 \x01(\t\"!\n\x06Status\x12\x0b\n\x07SUCCESS\x10\x00\x12\n\n\x06\x46\x41ILED\x10\x01\"\xa0\x01\n\x13GeneralBoolResponse\x12\x42\n\x06status\x18\x01 \x01(\x0e\x32\x32.fateark.proto.response.GeneralBoolResponse.Status\x12\x0f\n\x07payload\x18\x02 \x01(\x08\x12\x11\n\terror_msg\x18\x03 \x01(\t\"!\n\x06Status\x12\x0b\n\x07SUCCESS\x10\x00\x12\n\n\x06\x46\x41ILED\x10\x01\x42!Z\x1f./proto/gen/response;responsepbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'proto.response_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z\037./proto/gen/response;responsepb'
  _globals['_GENERALRESPONSE']._serialized_start=49
  _globals['_GENERALRESPONSE']._serialized_end=201
  _globals['_GENERALRESPONSE_STATUS']._serialized_start=168
  _globals['_GENERALRESPONSE_STATUS']._serialized_end=201
  _globals['_GENERALINT32RESPONSE']._serialized_start=204
  _globals['_GENERALINT32RESPONSE']._serialized_end=366
  _globals['_GENERALINT32RESPONSE_STATUS']._serialized_start=168
  _globals['_GENERALINT32RESPONSE_STATUS']._serialized_end=201
  _globals['_GENERALINT64RESPONSE']._serialized_start=369
  _globals['_GENERALINT64RESPONSE']._serialized_end=531
  _globals['_GENERALINT64RESPONSE_STATUS']._serialized_start=168
  _globals['_GENERALINT64RESPONSE_STATUS']._serialized_end=201
  _globals['_GENERALUINT64RESPONSE']._serialized_start=534
  _globals['_GENERALUINT64RESPONSE']._serialized_end=698
  _globals['_GENERALUINT64RESPONSE_STATUS']._serialized_start=168
  _globals['_GENERALUINT64RESPONSE_STATUS']._serialized_end=201
  _globals['_GENERALBOOLRESPONSE']._serialized_start=701
  _globals['_GENERALBOOLRESPONSE']._serialized_end=861
  _globals['_GENERALBOOLRESPONSE_STATUS']._serialized_start=168
  _globals['_GENERALBOOLRESPONSE_STATUS']._serialized_end=201
# @@protoc_insertion_point(module_scope)
